use commander_kernel::buffer::DynamicBuffer;
use criterion::{black_box, criterion_group, criterion_main, Criterion};

fn criterion_benchmark(c: &mut Criterion) {
    c.bench_function("Merge two small arrays", |b| {
        let mut buffer = DynamicBuffer::new();
        buffer.add(black_box(vec![0, 0, 0, 0]), black_box(4));
        buffer.add(black_box(vec![1, 1, 1, 1]), black_box(4));

        b.iter(|| buffer.merge())
    });

    c.bench_function("Merge two large arrays", |b| {
        let mut buffer = DynamicBuffer::new();
        buffer.add(black_box(vec![0u8; 1024 * 1024]), black_box(1024 * 1024));
        buffer.add(black_box(vec![1u8; 1024 * 1024]), black_box(1024 * 1024));

        b.iter(|| buffer.merge())
    });

    c.bench_function("Merge several large arrays", |b| {
        let mut buffer = DynamicBuffer::new();

        // 5 here is fine, because it is sampled a lot with criterion
        for _n in 1..=5 {
            buffer.add(black_box(vec![0u8; 1024 * 1024]), black_box(1024 * 1024));
        }

        b.iter(|| buffer.merge())
    });
}

criterion_group!(benches, criterion_benchmark);
criterion_main!(benches);
